#!/usr/bin/python3
#
#	PIC Metal Detector - Frequency Counting PIC Based Metal Detector
#	Copyright (C) 2013  Nicholas Tomlinson
#	
#	This program is free software: you can redistribute it and/or modify
#	it under the terms of the GNU General Public License as published by
#	the Free Software Foundation, either version 3 of the License, or
#	(at your option) any later version.
#	
#	This program is distributed in the hope that it will be useful,
#	but WITHOUT ANY WARRANTY; without even the implied warranty of
#	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#	GNU General Public License for more details.
#	
#	You should have received a copy of the GNU General Public License
#	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
#

acc_sample_alpha = 0.05
offs_sample_alpha = 0.001
window = 100.0

import sys

def ewma(acc_sample, alpha, new_sample):
	return acc_sample * (1.0 - alpha) + alpha * new_sample

with open('/dev/ttyUSB0', 'r') as rf, open('/dev/ttyUSB0', 'w') as wf:
	acc_sample = None
	offs_sample = None
	
	while True:
		try:
			wf.write('c\n')
			line = rf.readline().rstrip('\n')
			
			count = int(line, 16)
			sample = float(count)
			
			if acc_sample is None:
				acc_sample = sample
			if offs_sample is None:
				offs_sample = sample
			
			acc_sample = ewma(acc_sample, acc_sample_alpha, sample)
			offs_sample = ewma(offs_sample, offs_sample_alpha, sample)
			d = (acc_sample - offs_sample)/window + 0.5
			
			sys.stdout.write(str(d) + '\n')
			sys.stderr.write(line + ' ' + str(count) + ' ' + str(d) + '\n')
			sys.stdout.flush()
			sys.stderr.flush()
		except ValueError:
			pass
