#define __16f648a
#include <pic16f648a.h>

#include "eeprom.h"

/*
 * EEPROM Data:
 *     0x00: Count wait LSB
 *     0x01: Count wait MSB
 *     0x02: Referenec adjust LSB
 *     0x03: Referenec adjust MSB
 *     0x04: Count PWM on start up
 */
__code unsigned char __at 0x2100 __EEPROM[] =
{
	0x00,
	0x02,
	0x60,
	0x00,
	0x00
};

unsigned char eeprom_read_byte(unsigned char addr)
{
	EEADR = addr; //set the address to read from
	EECON1 = 1; //enable reading
	while (EECON1 & 1); //wait for it to be done
	EECON1 = 0; //reset the control register
	return EEDATA; //return the data
}

void eeprom_write_byte(unsigned char addr, unsigned char n)
{
	while (1){ //we might have to reattempt
		EEADR = addr; //set the address to read from
		EEDATA = n;
		EECON1 = 4; //enable writing
		
		//Follow the required sequence
		__asm
			movlw 0x55
			movwf EECON2
			movlw 0xAA
			movwf EECON2
			bsf EECON1, 1
		__endasm;
		
		while (EECON1 & 2); //wait for it to be done
		
		if (!(EECON1 & 8)){
			EECON1 = 0; //reset the control register
			return; //we're done
		}
		EECON1 = 0; //reset the control register
	}
}

unsigned int eeprom_read_int(unsigned char addr)
{
	return (unsigned int)eeprom_read_byte(addr) |
		((unsigned int)eeprom_read_byte(addr + 1) << 8);
}

void eeprom_write_int(unsigned char addr, unsigned int n)
{
	eeprom_write_byte(addr, n);
	eeprom_write_byte(addr + 1, n >> 8);
}

