/*
	PIC Metal Detector - Frequency Counting PIC Based Metal Detector
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

#define __16f648a
#include <pic16f648a.h>

void pwm_init(unsigned char period, unsigned int duty, unsigned char prescale)
{
	TRISB &= ~8; //Clear TRISB bit 3 as required by the PWM module
	
	//Set the period
	PR2 = period;
	
	//Set the duty cycle and enable the PWM module
	CCPR1L = duty >> 2;
	CCP1CON = 0xC | ((duty & 3) << 4);
	
	//Set up timer 2
	T2CON = _TMR2ON | (prescale & 3);
}

void pwm_deinit()
{
	PR2 = 0;
	CCPR1L = 0;
	CCP1CON = 0;
	T2CON = 0;
}
