/*
	PIC Metal Detector - Frequency Counting PIC Based Metal Detector
	Copyright (C) 2013  Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>. 
*/

#define __16f648a
#include <pic16f648a.h>

void serial_init()
{
	TRISB |= 6; //Set TRISB bits 2 and 1 as required by the USART
	TXSTA = _TXEN | _BRGH | _TRMT;
	RCSTA = _SPEN | _CREN;
	SPBRG = 25; //Baud rate of 9600 (4000000/(16 * 9600 - 1))
	PIE1 |= _RCIE; //Enable reception of the USART receive interrupt
}

void serial_transmit_char(char c)
{
	//Wait for TXREG to be available, then transmit
	while (!(PIR1 & _TXIF));
	TXREG = c;
}

void serial_transmit_string(__code const char *string)
{
	int i = 0;
	while (string[i]){
		if ('\n' == string[i]){
			serial_transmit_char('\r');
			serial_transmit_char('\n');
		}
		else {
			serial_transmit_char(string[i]);
		}
		i++;
	}
}

void serial_transmit_hex(unsigned char h)
{
	if (h < 10){
		serial_transmit_char('0' + h);
	}
	else {
		serial_transmit_char('A' + h - 10);
	}
}

void serial_transmit_byte(unsigned char b)
{
	unsigned char a = b >> 4;
	b = b & 0xF;
	serial_transmit_hex(a);
	serial_transmit_hex(b);
}

void serial_transmit_int(unsigned int i)
{
	int j = sizeof(int) << 3;
	while (j){
		j -= 8;
		serial_transmit_byte(i >> j);
	}
}

char serial_poll()
{
	//Remove overflow if necessary
	RCSTA = _SPEN;
	RCSTA = _SPEN | _CREN;
	
	//Determine whether the receiver has any data
	return PIR1 & _RCIF;
}

char serial_receive_char(char echo)
{
	char c = 0;
	
	//Remove overflow if necessary
	RCSTA = _SPEN;
	RCSTA = _SPEN | _CREN;
	
	//Wait to be able to receive, then grap the character
	while (!(PIR1 & _RCIF));
	c = RCREG;
	
	//Transmit a copy (better UI) and return
	if (echo){
		serial_transmit_char(c);
	}
	return c;
}

unsigned char serial_receive_hex()
{
	unsigned char i = 0;
	char c = serial_receive_char(1);
	
	if (c < '0'){
		return 0;
	}
	else if (c <= '9'){
		return c - '0';
	}
	else if (c < 'A'){
		return 0;
	}
	else if (c <= 'F'){
		return c + 10 - 'A';
	}
	else if (c < 'a'){
		return 0;
	}
	else if (c <= 'f'){
		return c + 10 - 'a';
	}
	else {
		return 0;
	}
}

unsigned char serial_receive_byte()
{
	return (serial_receive_hex() << 4) | serial_receive_hex();
}

unsigned int serial_receive_int()
{
	unsigned int i = 0;
	unsigned int r = 0;
	while (i < sizeof(unsigned int) << 1){
		r = (r << 4) | serial_receive_hex();
		i++;
	}
	return r;
}

